<?php

SugarAutoLoader::requireWithCustom('include/Sugarpdf/sugarpdf_config.php');

require_once('vendor/tcpdf/tcpdf.php');

class FlowchartPdfFile_lb60b614ad12af40_26596112 extends TCPDF{

	const PDF_MIME_TYPE = 'application/pdf';

	protected $documentName = '';
	protected $headerLogo = '';
	protected $headerTitle = '';
	protected $headerText = '';
	protected $footerText = '';
	protected $bodyHtml;

	function __construct($orientation = PDF_PAGE_ORIENTATION, $unit = PDF_UNIT, $format = PDF_PAGE_FORMAT, $unicode = true, $encoding = 'UTF-8', $diskcache = false){
		if(empty($encoding)){
			$encoding = "UTF-8";
		}
		parent::__construct($orientation, $unit, $format, $unicode, $encoding, $diskcache);
		$this->init();
	}

	public function setBodyHTML($bodyHTML){
		$this->bodyHtml = $bodyHTML;
	}

	public function setDocumentName($name){
		$this->documentName = $name;
	}

	public function setHeaderLogo($headerLogo){
		$this->headerLogo = $headerLogo;
	}

	public function setHeaderTitle($headerTitle){
		$this->headerTitle = $headerTitle;
	}

	public function setHeaderText($headerText){
		$this->headerText = $headerText;
	}

	public function setFooterText($footerText){
		$this->footerText = $footerText;
	}

	public function getDocumentName(){
		return $this->documentName;
	}

	public function getFileMimeType(){
		return self::PDF_MIME_TYPE;
	}

	public function process(){
		$this->prepareData();

		//turn off all error reporting so that PHP warnings don't munge the PDF code
		error_reporting(E_ALL);
		//TODO: Removed because of packageScan
		//set_time_limit(1800);

		//Create new page
		$this->AddPage();
		$this->SetFont(PDF_FONT_NAME_MAIN,'',8);
		$this->writeHTML($this->bodyHtml);
	}

	protected function init(){
		// set document information
		$this->SetCreator(PDF_CREATOR);
		$this->SetAuthor(PDF_AUTHOR);
		$this->SetTitle(PDF_TITLE);
		$this->SetSubject(PDF_SUBJECT);
		$this->SetKeywords(PDF_KEYWORDS);

		// set other properties
		$compression=false;
		if(PDF_COMPRESSION == "on"){
			$compression=true;
		}
		$this->SetCompression($compression);
		$protection=array();
		if(PDF_PROTECTION != ""){
			$protection=explode(",",PDF_PROTECTION);
		}

		$this->SetProtection($protection,blowfishDecode(blowfishGetKey('sugarpdf_pdf_user_password'), PDF_USER_PASSWORD),blowfishDecode(blowfishGetKey('sugarpdf_pdf_owner_password'), PDF_OWNER_PASSWORD));
		$this->setCellHeightRatio(K_CELL_HEIGHT_RATIO);
		$this->setJPEGQuality(intval(PDF_JPEG_QUALITY));
		$this->setPDFVersion(PDF_PDF_VERSION);

		// set default header data
		$this->setHeaderData(PDF_HEADER_LOGO, PDF_HEADER_LOGO_WIDTH, PDF_HEADER_TITLE, PDF_HEADER_STRING);

		// set header and footer fonts
		$this->setHeaderFont(Array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));
		$this->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));

		//set margins
		$this->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
		$this->setHeaderMargin(PDF_MARGIN_HEADER);
		$this->setFooterMargin(PDF_MARGIN_FOOTER);

		//set auto page breaks
		$this->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);

		//set image scale factor
		$this->setImageScale(PDF_IMAGE_SCALE_RATIO);

		$this->setPrintHeader(false);
		// always print page number at least
		$this->setPrintFooter(true);
	}

	protected function prepareData(){
		$headerLogo = '';
		if (!empty($this->headerLogo)) {
			if($this->headerLogo instanceof Note){
				$headerLogo = 'upload/tmp/' . $this->headerLogo->filename;
				//TODO:START: Removed because of packageScan
                $uploadFile = new UploadFile();
                $headerLogoNewLocation = 'tmp/' . $this->headerLogo->filename;
                $uploadFile->duplicate_file($this->headerLogo->id, $headerLogoNewLocation);
				//copy('upload/' . $this->headerLogo->id, $headerLogo);
				//TODO:END: Removed because of packageScan
			}
			//$headerLogo = 'upload/' . $pdfTemplate->id . $pdfTemplate->header_logo;
			//copy('upload/' . $pdfTemplate->id, $headerLogo);
		}
		if (!empty($headerLogo) || !empty($this->headerTitle) || !empty($this->headerText)){
			$this->setHeaderData(
				$headerLogo,
				PDF_HEADER_LOGO_WIDTH,
				$this->headerTitle,
				$this->headerText
			);
			$this->setPrintHeader(true);
		}

		if(empty($this->documentName)){
			$this->documentName = 'Pdf File ' . date('Ymd');
		}
		$cr = array(' ', "\r", "\n", "/");
		$this->documentName = str_replace($cr, '_', $this->documentName.".pdf");
	}

	public function saveFile($filename = ''/*, $dest = 'I'*/){
		if(empty($filename)){
			$filename = $this->documentName;
		}
		$filePath = 'upload://'.$filename;
		parent::Output($filePath, 'F');

//		$tmp = parent::Output('','S');
//		$badoutput = ob_get_contents();
//		if(strlen($badoutput) > 0) {
//			ob_end_clean();
//		}
//		file_put_contents('upload://'.$filename, ltrim($tmp));
		return $filename;
	}

	public function Header() {
		$ormargins = $this->getOriginalMargins();
		$headerfont = $this->getHeaderFont();
		$headerdata = $this->getHeaderData();

		if ($headerdata['logo'] && $headerdata['logo'] != K_BLANK_IMAGE) {
			$this->Image($headerdata['logo'], $this->GetX(), $this->getHeaderMargin(), $headerdata['logo_width'], 12);
			$imgy = $this->getImageRBY();
		} else {
			$imgy = $this->GetY();
		}
		$cell_height = round(($this->getCellHeightRatio() * $headerfont[2]) / $this->getScaleFactor(), 2);
		// set starting margin for text data cell
		if ($this->getRTL()) {
			$header_x = $ormargins['right'] + ($headerdata['logo_width'] * 1.1);
		} else {
			$header_x = $ormargins['left'] + ($headerdata['logo_width'] * 1.1);
		}
		$this->SetTextColor(0, 0, 0);
		// header title
		$this->SetFont($headerfont[0], 'B', $headerfont[2] + 1);
		$this->SetX($header_x);
		$this->Cell(0, $cell_height, $headerdata['title'], 0, 1, '', 0, '', 0);
		// header string
		$this->SetFont($headerfont[0], $headerfont[1], $headerfont[2]);
		$this->SetX($header_x);
		$this->MultiCell(0, $cell_height, $headerdata['string'], 0, '', 0, 1, '', '', true, 0, false);
		// print an ending header line
		$this->SetLineStyle(array('width' => 0.85 / $this->getScaleFactor(), 'cap' => 'butt', 'join' => 'miter', 'dash' => 0, 'color' => array(0, 0, 0)));
		$this->SetY((2.835 / $this->getScaleFactor()) + max($imgy, $this->GetY()));
		if ($this->getRTL()) {
			$this->SetX($ormargins['right']);
		} else {
			$this->SetX($ormargins['left']);
		}
		$this->Cell(0, 0, '', 'T', 0, 'C');
	}

	/**
	 * PDF manager specific Footer function
	 */
	public function Footer() {
		$cur_y = $this->GetY();
		$ormargins = $this->getOriginalMargins();
		$this->SetTextColor(0, 0, 0);
		//set style for cell border
		$line_width = 0.85 / $this->getScaleFactor();
		$this->SetLineStyle(array('width' => $line_width, 'cap' => 'butt', 'join' => 'miter', 'dash' => 0, 'color' => array(0, 0, 0)));
		//print document barcode
		$barcode = $this->getBarcode();
		if (!empty($barcode)) {
			$this->Ln($line_width);
			$barcode_width = round(($this->getPageWidth() - $ormargins['left'] - $ormargins['right'])/3);
			$this->write1DBarcode($barcode, 'C128B', $this->GetX(), $cur_y + $line_width, $barcode_width, (($this->getFooterMargin() / 3) - $line_width), 0.3, '', '');
		}
		if (empty($this->pagegroups)) {
			$pagenumtxt = $this->l['w_page'].' '.$this->getAliasNumPage().' / '.$this->getAliasNbPages();
		} else {
			$pagenumtxt = $this->l['w_page'].' '.$this->getPageNumGroupAlias().' / '.$this->getPageGroupAlias();
		}
		$this->SetY($cur_y);

		if ($this->getRTL()) {
			$this->SetX($ormargins['right']);
			if ($this->footerText) {
				// footer text and page number
				$this->Cell(0, 0, $this->footerText, 'T', 0, 'R');
				$this->Cell(0, 0, $pagenumtxt, 0, 0, 'L');
			} else {
				// page number only
				$this->Cell(0, 0, $pagenumtxt, 'T', 0, 'L');
			}
		} else {
			$this->SetX($ormargins['left']);
			if ($this->footerText) {
				// footer text and page number
				$this->Cell(0, 0, $this->footerText, 'T', 0, 'L');
				$this->Cell(0, 0, $pagenumtxt, 0, 0, 'R');
			} else {
				// page number only
				$this->Cell(0, 0, $pagenumtxt, 'T', 0, 'R');
			}
		}
	}

}
