(function(app){

    app.events.on('app:sync:complete', function(){

        let flowchartValidation_lb60b614ad12af40_26596112 = {

            validationName: 'lb_validation_lb60b614ad12af40_26596112',

            alertErrorName: 'lb_validation_errors_Contacts',

            modelHasLbValidationFunc: function(model){
                return !_.isEmpty(model['lbValidations']) && !_.isEmpty(model['lbValidations'][this.validationName]);
            },

            isValidModule: function(module) {
                return module == 'Contacts';
            },

            isModelValidForLbValidationFunc: function(model, module){
                module = module || model.get('module') || model.get('_module');
                return module == 'Contacts';
            },

            addLbValidationFunc: function(model, module, viewFields){
                var self = this;

                model.addValidationTask(this.validationName, function (fields, errors, callback) {

                    let url = app.api.buildURL('flowchart/validate/lb60b614ad12af40_26596112', '', {}, {});
                    let data = {};
                    data['entity_attributes'] = model.attributes;
                    data['previous_entity_attributes'] = model.getSynced();
                    data['changed_attributes'] = model.changedAttributes(model.getSynced());
                    data['is_new_record'] = model.isNew();

                    app.api.call('create', url, data, {
                        success: _.bind(function (validationErrors) {
                            if (!_.isEmpty(validationErrors)) {

                                let errorsToAlertList = [];

                                _.each(validationErrors, function (validationError, key) {
                                    let errorKey = 'lb_v_error_lb60b614ad12af40_26596112_' + key,
                                        fieldName = validationError['targetField'],
                                        errorText = validationError['errorText'],
                                        operatorId = validationError['operatorId'];

                                    if(!_.isString(fieldName) || _.isEmpty(fieldName) || !_.isString(errorText) || _.isEmpty(errorText)){
                                        return;
                                    }

                                    app.error.errorName2Keys[errorKey] = errorText;

                                    errors[fieldName] = errors[fieldName] || {};
                                    errors[fieldName][errorKey] = true;

                                    if(_.indexOf(viewFields, fieldName) == -1){
                                        errorsToAlertList.push({
                                            'fieldName': fieldName,
                                            'errorText': errorText,
                                            'operatorId': operatorId
                                        });
                                    }
                                });

                                if(!_.isEmpty(errorsToAlertList)){
                                    var alertComponent = self.getErrorAlertComponent();
                                    _.each(errorsToAlertList, function(errorsToAlertItem){
                                        self.addErrorToAlertList(alertComponent, errorsToAlertItem.fieldName, errorsToAlertItem.errorText, errorsToAlertItem.operatorId);
                                    });
                                    self.addErrorHtmlListToAlert(alertComponent, model);
                                }
                            }
                            callback(null, fields, errors);
                        }),
                        error: function() {
                            callback(null, fields, errors);
                        }
                    });

                    model['lbValidations'] = model['lbValidations'] || {};
                    model['lbValidations'][self.validationName] = true;
                });
            },

            getErrorAlertComponent: function(){

                var alertName = this.alertErrorName, // 'invalid-data',
                    alertComponent = app.alert.get(alertName);

                if(_.isEmpty(alertComponent)) {
                    alertComponent = app.alert.show(alertName, {
                        level: 'error',
                        messages: 'In order to be saved, this record should meet the following requirements'
                    });
                }else{
                    alertComponent.$el.find('.lb-validation-errors').remove();
                }

                alertComponent['lbErrorsList'] = alertComponent['lbErrorsList'] || {};
                alertComponent['lbErrorsList'][this.validationName] = {};

                return alertComponent;
            },

            addErrorToAlertList: function(alertComponent, fieldName, errorText, validationOpId){

                // alertComponent['lbErrorsList'] = alertComponent['lbErrorsList'] || {};
                // alertComponent['lbErrorsList'][this.validationName] = alertComponent['lbErrorsList'][this.validationName] || {};

                alertComponent['lbErrorsList'][this.validationName][validationOpId] = {
                    'fieldName': fieldName,
                    'errorText': errorText
                };
            },

            addErrorHtmlListToAlert: function(alertComponent, model){

                var
                    self = this,
                    errorAlertEl = alertComponent.$el,
                    errorsList = alertComponent['lbErrorsList'],
                    allErrorsList = {},
                    errorsListBlock;

                if(!_.isEmpty(errorsList)){
                    _.each(errorsList, function(errorFlowchartGroup){
                        _.each(errorFlowchartGroup, function(errorGroup){
                            var fieldName = errorGroup.fieldName;
                            allErrorsList[fieldName] = allErrorsList[fieldName] || [];
                            allErrorsList[fieldName].push(errorGroup);
                        });
                    });
                }

                if(!_.isEmpty(allErrorsList)){
                    errorsListBlock = $(
                        '<span class="lb-validation-errors" style="display: block; margin-top: 10px;">' +
                            '<a href="javascript:void(0)" class="collapsed">Show errors list</a>' +
                            '<span class="lb-errors-list" style="display:none; font-size: 13px;">' +
                            '</span>' +
                        '</span>'
                    );

                    !_.each(allErrorsList, function(error, fieldName){
                        var html =
                            '<span style="display:block">' +
                                '<span style="font-weight: bold;">' +
                                    self.getFieldLabel(fieldName, model) +
                                '</span>: ';

                        _.each(error, function(fieldErrors){
                            html += '<span style="display:block">' + fieldErrors.errorText + '<span>';
                        });
                        // errorsListBlock.find('.lb-errors-list').append(
                        //     '<span style="display:block">' +
                        //         '<span style="font-weight: bold;">' +
                        //             self.getFieldLabel(error.fieldName, model) +
                        //         '</span>: ' +
                        //         error.errorText +
                        //     '</span>'
                        // );
                        html += '</span>';

                        errorsListBlock.find('.lb-errors-list').append(html);
                    });

                    errorAlertEl.find('.text').append(errorsListBlock);
                    errorAlertEl.find('.lb-validation-errors a').click(function() {
                        if($(this).hasClass('collapsed')){
                            $(this).removeClass('collapsed').html('Hide errors list');
                            // errorAlertEl.find('.lb-errors-list').slideDown( "slow", function() {});
                        }else{
                            $(this).addClass('collapsed').html('Show errors list');
                            // errorAlertEl.find('.lb-errors-list').slideUp( "slow", function() {});
                        }
                         errorAlertEl.find('.lb-errors-list').toggle();
                    });
                }else{
                    errorAlertEl.find('.text .lb-validation-errors').html('');
                }
            },

            getFieldLabel: function(fieldName, model){
                var fieldDefs = model.fields[fieldName] || {},
                    fieldLabelKey = fieldDefs['vname'] || fieldName,
                    moduleName = 'Contacts';

                return app.lang.get(fieldLabelKey, moduleName);
            },

            addLbValidationToCollectionFunc: function(collection, module, viewAvailableFields){
                if(_.has(collection, 'lbValidations') && collection.lbValidations[this.validationName]){
                    return;
                }

                var self = this;

                collection['lbValidations'] = collection['lbValidations'] || {};
                collection['lbValidations'][this.validationName] = true;
                collection['lbAvailableFieldsForValidation'] = viewAvailableFields;

                self.addValidationToModels(collection, module);
                collection.once("sync", function () {
                    self.addValidationToModels(this, module);
                });
            },

            addValidationToModels: function(collection, module){
                var self = this;

                _.each(collection.models, function (model) {
                    if (self.isModelValidForLbValidationFunc(model, module) && !self.modelHasLbValidationFunc(model)) {
                        let fields = model.collection['lbAvailableFieldsForValidation'] || [];
                        self.addLbValidationFunc(model, module, fields);
                    }
                });

                collection.on("add", function (model) {
                    if (self.isModelValidForLbValidationFunc(model, module) && !self.modelHasLbValidationFunc(model)) {
                        let fields = model.collection['lbAvailableFieldsForValidation'] || [];
                        self.addLbValidationFunc(model, module, fields);
                    }
                });

                collection.on("reset", function () {
                    _.each(collection.models, function (model) {
                        if (self.isModelValidForLbValidationFunc(model,module) && !self.modelHasLbValidationFunc(model)) {
                            let fields = model.collection['lbAvailableFieldsForValidation'] || [];
                            self.addLbValidationFunc(model,module, fields);
                        }
                    });
                });
            },

            findTileView: function (components) {
                const self = this;
                if (!components) {
                    return null;
                }
                let tileView = components.find(function(component) {
                    return (_.has(component, 'name') && (component.name == 'pipeline-recordlist-content'));
                });
                if (tileView) {
                    return tileView;
                }
                for (let i = 0; i < components.length; i++) {
                    let component = components[i];
                    if (_.has(component, '_components')) {
                        tileView = self.findTileView(component._components);
                        if (tileView) {
                            return tileView;
                        }
                    }
                }
                return null;
            },

            addValidationToTileViewModels: function (layout, module){
                let self = this;
                if (!self.isValidModule(module)) {
                    return;
                }
                if (!layout || !_.has(layout, '_components')) {
                    return;
                }
                let tileView = self.findTileView(layout._components);
                if (!tileView) {
                    return;
                }
                let oldSaveModel = tileView.saveModel;
                tileView.saveModel = function(model, pipelineData){
                    if(self.isModelValidForLbValidationFunc(model, module) && !self.modelHasLbValidationFunc(model)) {
                        self.addLbValidationFunc(model, module, []);
                    }
                    oldSaveModel.apply(tileView, arguments);
                };
            },


            checkLayoutComponentsForModels: function (components){
                var self = this;
                _.each(components, function(component){
                    let module = component.module;

                    if(
                        (_.has(component, 'action') && (component.action == 'list' || component.action == 'detail' || component.action == 'create')) ||
                        (_.has(component, 'type') && component.type == 'create')
                    ){
                        let fieldsList;

                        if(_.has(component, 'metaFields') && !_.isEmpty(component.metaFields) ){
                            fieldsList = component.metaFields;
                        }else if(_.has(component, 'fields') && !_.isEmpty(component.fields)){
                            fieldsList = component.fields;
                        }

                        if(!_.isEmpty(fieldsList) ){
                            let fields = [];
                            _.each(fieldsList, function (field) {
                                if (_.isObject(field) && _.has(field, 'name') && !_.isEmpty(field.name)) {
                                    if(_.has(field, 'fields') && field.fields){
                                        _.each(field.fields, function(fieldSetField) {
                                            if(_.has(fieldSetField, 'name') && !_.isEmpty(fieldSetField.name))
                                                fields.push(fieldSetField.name);
                                        });
                                    }
                                    fields.push(field.name);

                                }
                            });

                            if(!_.isEmpty(fields) && _.has(component, 'collection')) {
                                self.addLbValidationToCollectionFunc(component.collection, module, fields);
                            }
                        }
                    }

                    let subComponents = component._components;
                    if(!_.isEmpty(subComponents)) {
                        self.checkLayoutComponentsForModels(subComponents);
                    }
                });
            }

        };

        app.router.on('route:create', function(module){

            if (!_.isEmpty(app.drawer) && _.has(app.drawer, '_components')) {
                let subComponents = app.drawer._components;
                flowchartValidation_lb60b614ad12af40_26596112.checkLayoutComponentsForModels(subComponents, true);
            }

            let createModel = app.controller.context.get('model');
            if(flowchartValidation_lb60b614ad12af40_26596112.isModelValidForLbValidationFunc(createModel, module) && !flowchartValidation_lb60b614ad12af40_26596112.modelHasLbValidationFunc(createModel)) {
                flowchartValidation_lb60b614ad12af40_26596112.addLbValidationFunc(createModel, module);
            }
        });

        app.router.on('route', function(module) {
            if (!_.isEmpty(app.controller.layout) && _.has(app.controller.layout, '_components')) {
                let subComponents = app.controller.layout._components;
                flowchartValidation_lb60b614ad12af40_26596112.checkLayoutComponentsForModels(subComponents);
            }

            let collection = app.controller.context.get('collection');
            flowchartValidation_lb60b614ad12af40_26596112.addLbValidationToCollectionFunc(collection, null, []);


            if(!_.isEmpty(app.controller.context.children)) {
                _.each(app.controller.context.children, function(child){
                    if(!_.isEmpty(child.get('collection'))) {
                        let collection = child.get('collection');
                        flowchartValidation_lb60b614ad12af40_26596112.addLbValidationToCollectionFunc(collection, null, []);
                    }
                });
            }
        });

        app.events.on('app:view:change', function(layout, params){
            if (layout == 'create' && params && _.has(params, 'module') && _.has(params, 'model') && params.model) {
                if(flowchartValidation_lb60b614ad12af40_26596112.isModelValidForLbValidationFunc(params.model, params.module) && !flowchartValidation_lb60b614ad12af40_26596112.modelHasLbValidationFunc(params.model)) {
                    flowchartValidation_lb60b614ad12af40_26596112.addLbValidationFunc(params.model, params.module);
                }
            }

            if (layout == 'pipeline-records'  && params && _.has(params, 'module')) {
                let module = params.module;
                flowchartValidation_lb60b614ad12af40_26596112.addValidationToTileViewModels(app.controller.layout, module);
            }

            if (!_.isEmpty(app.controller.layout) && _.has(app.controller.layout, '_components')) {
                let subComponents = app.controller.layout._components;
                flowchartValidation_lb60b614ad12af40_26596112.checkLayoutComponentsForModels(subComponents);
            }

            let collection = app.controller.context.get('collection');
            flowchartValidation_lb60b614ad12af40_26596112.addLbValidationToCollectionFunc(collection, null, []);


            if(!_.isEmpty(app.controller.context.children)) {
                _.each(app.controller.context.children, function(child){
                    if(!_.isEmpty(child.get('collection'))) {
                        let collection = child.get('collection');
                        flowchartValidation_lb60b614ad12af40_26596112.addLbValidationToCollectionFunc(collection, null, []);
                    }
                });
            }
        });

    });
})(SUGAR.App);
